@@ -27,9 +27,10 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define _LARGEFILE_SOURCE 1
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
-#include "mpg321.h"
 
 #include <unistd.h>
 #include <sys/mman.h>
@@ -41,6 +42,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
+#include "mpg321.h"
+
 unsigned long current_frame=0;
 
 enum mad_flow read_from_mmap(void *data, struct mad_stream *stream)
@@ -53,7 +56,7 @@ enum mad_flow read_from_mmap(void *data, struct mad_stream *stream)
     if (status != MPG321_REWINDING && playbuf->done)
     {
         status = MPG321_STOPPED;
-        if (options.opt & MPG321_REMOTE_PLAY) printf("@P 0\n");
+        if (options.opt & MPG321_REMOTE_PLAY) printf("@P 3\n");
         return MAD_FLOW_STOP;
     }
 
@@ -87,6 +90,7 @@ enum mad_flow read_from_fd(void *data, struct mad_stream *stream)
 {
     buffer *playbuf = data;
     int bytes_to_preserve = stream->bufend - stream->next_frame;
+    int readbytes;
     
     if(playbuf->done)
     {
@@ -108,12 +112,18 @@ enum mad_flow read_from_fd(void *data, struct mad_stream *stream)
     /* need to preserve the bytes which comprise an incomplete
        frame, that mad has passed back to us */
     if (bytes_to_preserve)
+    {
         memmove(playbuf->buf, stream->next_frame, bytes_to_preserve);
+    }
 
-    if( !(read(playbuf->fd, playbuf->buf + bytes_to_preserve, BUF_SIZE - bytes_to_preserve) > 0) )
+
+    readbytes = read(playbuf->fd, playbuf->buf + bytes_to_preserve,
+		     BUF_SIZE - bytes_to_preserve);
+    if (readbytes <= 0) {
         playbuf->done = 1;
-
-    mad_stream_buffer(stream, playbuf->buf, playbuf->length);
+    } else {
+        mad_stream_buffer(stream, playbuf->buf, readbytes+bytes_to_preserve);
+    }
     
     return MAD_FLOW_CONTINUE;
 }    
@@ -171,6 +181,7 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
 {
     char long_currenttime_str[14]; /* this *will* fill if you're using 100000+ minute mp3s */
     char long_remaintime_str[14];
+    static int frames_played = 0;
     
     buffer *playbuf = (buffer *)data;
     mad_timer_t time_remaining;
@@ -178,6 +189,7 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
     if (stop_playing_file)
     {
         stop_playing_file = 0;
+        status = MPG321_STOPPED;
         return MAD_FLOW_STOP;
     }
     
@@ -191,10 +203,11 @@ enum mad_flow read_header(void *data, struct mad_header const * header)
     }
 
     /* Stop playing if -n is used, and we're at the frame specified. */
-    if ((playbuf->max_frames != -1) && (current_frame > playbuf->max_frames))
+    if ((playbuf->max_frames != -1) && (frames_played++ > playbuf->max_frames))
     {
-      status = MPG321_STOPPED;
-      return MAD_FLOW_STOP;
+        frames_played = 0;
+        status = MPG321_STOPPED;
+        return MAD_FLOW_STOP;
     }
 
     current_frame++;
@@ -447,20 +460,11 @@ void scan(void const *ptr, ssize_t len, buffer *buf)
             break;
         }
             
-       /* In quiet mode, don't scan the whole file for length since
-           it takes so long on slow file systems like sshfs. cplay
-           calls mpg321 for playback in quiet mode. */
-        if ((options.opt & MPG321_QUIET_PLAY) && (buf->num_frames > 20))
-        {
-            break;
-        }
-
         mad_timer_add(&buf->duration, header.duration);
     }
 
     if (!is_vbr)
     {
-       if (header.bitrate!=0 && MAD_NSBSAMPLES(&header)!=0) {
         double time = (len * 8.0) / (header.bitrate); /* time in seconds */
         double timefrac = (double)time - ((long)(time));
         long nsamples = 32 * MAD_NSBSAMPLES(&header); /* samples per frame */
@@ -469,7 +473,6 @@ void scan(void const *ptr, ssize_t len, buffer *buf)
         buf->num_frames = (long) (time * header.samplerate / nsamples);
 
         mad_timer_set(&buf->duration, (long)time, (long)(timefrac*100), 100);
-       }
     }
         
     else if (has_xing)
@@ -559,7 +562,13 @@ enum mad_flow move(buffer *buf, signed long frames)
             current_frame = 0;
         else
             current_frame += frames;
-        current_time = buf->times[current_frame];
+        if (current_frame > 0)
+            current_time = buf->times[current_frame];
+        else
+        {
+            current_time.seconds = 0;
+            current_time.fraction = 0;
+        }
         return MAD_FLOW_STOP;
     }
 
@@ -759,7 +768,7 @@ enum mad_flow output(void *data,
     {
         while (nsamples--)
         {
-            tempsample = (mad_fixed_t)((*left_ch++ * (double)options.volume)/MAD_F_ONE);
+            tempsample = mad_f_mul(*left_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
 
 #ifndef WORDS_BIGENDIAN
@@ -770,8 +779,9 @@ enum mad_flow output(void *data,
             *ptr++ = (unsigned char) (sample >> 0);
 #endif
             
-            tempsample = (mad_fixed_t)((*right_ch++ * (double)options.volume)/MAD_F_ONE);
+            tempsample = mad_f_mul(*right_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
+
 #ifndef WORDS_BIGENDIAN
             *ptr++ = (unsigned char) (sample >> 0);
             *ptr++ = (unsigned char) (sample >> 8);
@@ -788,7 +798,7 @@ enum mad_flow output(void *data,
     {
         while (nsamples--)
         {
-            tempsample = (mad_fixed_t)((*left_ch++ * (double)options.volume)/MAD_F_ONE);
+            tempsample = mad_f_mul(*left_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
             
             /* Just duplicate the sample across both channels. */
@@ -812,7 +822,7 @@ enum mad_flow output(void *data,
     {
         while (nsamples--)
         {
-            tempsample = (mad_fixed_t)((*left_ch++ * (double)options.volume)/MAD_F_ONE);
+            tempsample = mad_f_mul(*left_ch++, options.volume);
             sample = (signed int) audio_linear_dither(16, tempsample, &dither);
             
 #ifndef WORDS_BIGENDIAN
