@@ -25,9 +25,11 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define _LARGEFILE_SOURCE 1
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 
-#include "mpg321.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -46,9 +48,24 @@
 #include <sys/mman.h>
 
 #include "getopt.h" /* GNU getopt is needed, so I included it */
+#include "mpg321.h"
 
 #include <id3tag.h>
 
+/*xterm title setting -- (c) Copyright Nanakos Chrysostomos  06/02/2005 (nanakos@wired-net.gr)*/
+#include <termios.h>
+#include <config.h>
+
+FILE *ctty, *tty_in, *tty_out;
+int TTY_FILENO;
+
+struct termios tty_ts, tty_ts_orig; /* termios*/
+struct termios *tty_ts_orig_pt = NULL;
+static char temp[1024];
+char *ctty_path();
+
+
+
 int shuffle_play;
 int stop_playing_file = 0;
 int quit_now = 0;
@@ -64,7 +81,8 @@ char *id3_get_tag (struct id3_tag const *tag, char const *what, unsigned int max
 void mpg123_boilerplate()
 {
     fprintf(stderr,"High Performance MPEG 1.0/2.0/2.5 Audio Player for Layer 1, 2, and 3.\n"
-        "Version " FAKEVERSION " (" VERSIONDATE "). Written and copyrights by Joe Drew.\n"
+        "Version " FAKEVERSION " (" VERSIONDATE "). Written and copyrights by Joe Drew,\n"
+	"now maintained by Nanakos Chrysostomos and others.\n"
         "Uses code from various people. See 'README' for more!\n"
         "THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY! USE AT YOUR OWN RISK!\n");
 }
@@ -86,9 +104,9 @@ void usage(char *argv0)
         "   --quiet or -q            Quiet mode (no title or boilerplate)\n"
         "   --gain N or -g N         Set gain (audio volume) to N (0-100)\n"
         "   --skip N or -k N         Skip N frames into the file\n"
-        "   --verbose or -v          Be more verbose in playing files\n"
+        "   --frames N or -n N       Play only the first N frames\n"
         "   -o dt                    Set output devicetype to dt\n" 
-    "                                [esd,alsa(09),arts,sun,oss]\n"
+        "                                [esd,alsa(09),arts,sun,oss]\n"
         "   --audiodevice N or -a N  Use N for audio-out\n"
         "   --stdout or -s           Use stdout for audio-out\n"
         "   --au N                   Use au file N for output\n"
@@ -98,7 +116,12 @@ void usage(char *argv0)
         "   --list N or -@ N         Use playlist N as list of MP3 files\n"
         "   --random or -Z           Play files randomly until interrupted\n"
         "   --shuffle or -z          Shuffle list of files before playing\n"
+        "   --loop N or -l N         Play files N times. 0 means until\n"
+        "                            interrupted\n"
         "   -R                       Use remote control interface\n"
+        "   -p hostname:port         Use proxy server\n"
+        "   -u username:password     Use proxy server basic authentication\n"
+        "   -U username:password     Use proxy server basic authentication by using environment variables\n"
         "   --aggressive             Try to get higher priority\n"
         "   --help or --longhelp     Print this help screen\n"
         "   --version or -V          Print version information\n"
@@ -238,6 +261,8 @@ int main(int argc, char *argv[])
         exit(1);
     }
 
+    loop_remaining = 1;
+
     options.volume = MAD_F_ONE;
 
     status = MPG321_PLAYING;
@@ -278,11 +303,12 @@ int main(int argc, char *argv[])
     {
         printf ("@R MPG123\n");
     }
-    
+    tty_control();    
     /* Play the mpeg files or zip it! */
     while((currentfile = get_next_file(pl, &playbuf)))
     {
-        if (quit_now) 
+        //printf("Current File: %s\n",currentfile);
+	   if (quit_now) 
             break;
         
         signal(SIGINT, SIG_DFL);
@@ -292,11 +318,12 @@ int main(int argc, char *argv[])
         playbuf.length = 0;
         playbuf.done = 0;
         playbuf.num_frames = 0;
+        current_frame = 0;
         playbuf.max_frames = -1;
         strncpy(playbuf.filename,currentfile, PATH_MAX);
         playbuf.filename[PATH_MAX-1] = '\0';
         
-        if (status == MPG321_PLAYING) 
+        if (status == MPG321_PLAYING || status == MPG321_STOPPED) 
             file_change = 1;
 
         mad_timer_reset(&playbuf.duration);
@@ -413,14 +440,12 @@ int main(int argc, char *argv[])
             
             if(fstat(fd, &stat) == -1)
             {
-                close(fd);
                 mpg321_error(currentfile);
                 continue;
             }
             
             if (!S_ISREG(stat.st_mode))
             {
-                close(fd);
                 continue;
             }
             
@@ -437,12 +462,10 @@ int main(int argc, char *argv[])
             if((playbuf.buf = mmap(0, playbuf.length, PROT_READ, MAP_SHARED, fd, 0))
                                 == MAP_FAILED)
             {
-                close(fd);
                 mpg321_error(currentfile);
                 continue;
             }
             
-            close(fd);
             playbuf.frames[0] = playbuf.buf;
             
             mad_decoder_init(&decoder, &playbuf, read_from_mmap, read_header, /*filter*/0,
@@ -478,17 +501,23 @@ int main(int argc, char *argv[])
             }
 
             fprintf(stderr,"Playing MPEG stream from %s ...\n", basen);
-    
+            
+			/*Printing xterm title*/
+			osc_print(0,0,basen);
+	    
             free(dirc);
             free(basec);
         }    
 
         signal(SIGINT, handle_signals);
-
+        /*Give control back so that we can implement SIG's*/
+        set_tty_restore();
         /* Every time the user gets us to rewind, we exit decoding,
            reinitialize it, and re-start it */
+      
         while (1)
         {
+            decoder.options |= MAD_OPTION_IGNORECRC;
             mad_decoder_run(&decoder, MAD_DECODER_MODE_SYNC);
             
             /* if we're rewinding on an mmap()ed stream */
@@ -516,9 +545,6 @@ int main(int argc, char *argv[])
 
         mad_decoder_finish(&decoder);
 
-        if (quit_now)
-            break;
-
         if (playbuf.frames)
              free(playbuf.frames);
 
@@ -528,6 +554,7 @@ int main(int argc, char *argv[])
         if (playbuf.fd == -1)
         {
             munmap(playbuf.buf, playbuf.length);
+            close(fd);
         }
 
         else
@@ -542,10 +569,12 @@ int main(int argc, char *argv[])
         ao_close(playdevice);
 
     ao_shutdown();
+    /*Restoring TTY*/
+    set_tty_restore();
+    osc_print(0,0,"Terminal");
+    if (ctty)
+    fclose(ctty);
 
-#if defined(RAW_SUPPORT) || defined(HTTP_SUPPORT) || defined(FTP_SUPPORT) 
-    if(fd) close(fd);
-#endif
 
     return(0);
 }
@@ -639,3 +668,139 @@ char *id3_get_tag (struct id3_tag const *tag, char const *what, unsigned int max
 
     return (retval);
 }
+
+int tty_control()
+{
+    ctty = NULL;
+
+    if (!isatty(STDIN_FILENO) || !isatty(STDOUT_FILENO)) {
+      /* get controlling terminal */
+      char *program_name = "mpg321";
+      if ((ctty = fopen((char *)ctty_path(), "r+")) == NULL) {
+	fprintf(stderr, "%s: failed to get controlling terminal\n",program_name );
+        exit(EXIT_FAILURE);
+      }
+    }
+    if (!isatty(STDIN_FILENO) && ctty) {
+        tty_in = ctty;
+        TTY_FILENO = fileno(tty_in);
+    } else {
+        tty_in = stdin;
+        TTY_FILENO = STDIN_FILENO;
+    }
+    if (!isatty(STDOUT_FILENO) && ctty)
+        tty_out = ctty;
+    else
+        tty_out = stdout;
+
+    set_tty_raw();
+    return 0;
+	
+}
+
+int set_tty_raw()
+{
+    /* get and backup tty_in termios */
+    tcgetattr(TTY_FILENO, &tty_ts);
+    tty_ts_orig = tty_ts;
+    tty_ts_orig_pt = &tty_ts_orig;
+
+    /* set tty raw */
+    tty_ts.c_iflag = 0;
+    tty_ts.c_lflag = 0;
+
+    tty_ts.c_cc[VMIN] = 1;
+    tty_ts.c_cc[VTIME] = 1;
+    tty_ts.c_lflag &= ~(ICANON | ECHO);
+    tcsetattr(TTY_FILENO, TCSANOW, &tty_ts);
+    return 0;
+}
+
+int set_tty_restore(void)
+{
+    /* restore tty mode */
+    if (tty_ts_orig_pt)
+	tcsetattr(TTY_FILENO, TCSAFLUSH, tty_ts_orig_pt);
+    return 0;
+}
+
+
+/* issue raw escape sequence */
+int raw_print(char *ctlseq)
+{
+   int c;
+   while ((c = *ctlseq++)) {
+      if (c == '\\' && *ctlseq) {
+         switch (c = *ctlseq++) {
+            case 'e':
+               c = '\033';
+               break;
+            case 'a':
+               c = '\007';
+               break;
+            case 'b':
+               c = '\b';
+               break;
+            case 'f':
+               c = '\f';
+               break;
+            case 'n':
+               c = '\n';
+               break;
+            case 'r':
+               c = '\r';
+               break;
+            case 't':
+               c = '\t';
+               break;
+            case 'v':
+               c = (int) 0x0B;
+               break;
+            case '0':
+            case '1':
+            case '2':
+            case '3':
+            case '4':
+            case '5':
+            case '6':
+            case '7':
+               c -= '0';
+               if (*ctlseq >= '0' && *ctlseq <= '7')
+                  c = (c * 8) + (*ctlseq++ - '0');
+               if (*ctlseq >= '0' && *ctlseq <= '7')
+                  c = (c * 8) + (*ctlseq++ - '0');
+               break;
+            case '\\':
+               break;
+            default:
+               fputc('\\', tty_out);
+               break;
+         }
+      }
+  fputc(c, tty_out);
+   }
+   fflush(tty_out);
+   return 0;
+}
+
+int osc_print(int ps1, int ps2, char* pt)
+{
+
+    if (pt && *pt)
+	snprintf(temp, sizeof(temp), "\033]%d;%s\007", ps1, pt);
+    else
+        snprintf(temp, sizeof(temp), "\033]%d;?\007", ps1);
+   raw_print(temp);
+   return 0;
+}
+
+/* find path to controlling terminal */
+char *ctty_path()
+{
+   int i;
+   char* tty_path = NULL;
+   for (i = 0; i <= 2; i++)
+       if ((tty_path = ttyname(i)))
+                break;
+   return tty_path;
+}
