@@ -38,8 +38,8 @@
 #include <ao/ao.h>
 #include <mad.h>
 
-#define FAKEVERSION "0.59q"
-#define VERSIONDATE "2002/03/23"
+#define FAKEVERSION "0.2.11"
+#define VERSIONDATE "2009/06/16"
 
 /* playlist structure */
 typedef struct pl
@@ -95,7 +95,7 @@ typedef struct
     char *device;
     signed long seek;
     signed long maxframes;
-    int volume;
+    mad_fixed_t volume;
     int skip_printing_frames;
 } mpg321_options;    
 
@@ -109,6 +109,7 @@ extern char *playlist_file;
 extern int quit_now;
 extern char remote_input_buf[PATH_MAX + 5];
 extern int file_change;
+int loop_remaining;
 
 extern int status;
 
@@ -195,4 +196,13 @@ void usage(char *);
 
 RETSIGTYPE handle_signals(int sig) ;
 
+/* xterm functions */
+int tty_control();
+int set_tty_raw();
+int set_tty_restore();
+int raw_print(char *ctlseq);
+int osc_print(int ps1,int ps2,char *pt);
+char *ctty_path();
+
+
 #endif /* _MPG321_H_ */
