@@ -25,10 +25,12 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define _LARGEFILE_SOURCE 1
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 
 #include "mpg321.h"
-
 #include "getopt.h"
 
 #include <sys/time.h>
@@ -36,6 +38,15 @@
 #include <unistd.h>
 #include <string.h>
 
+
+/* Nanakos Chrysostomos (c) Copyright 2006 (nanakos@wired-net.gr)*/
+extern int proxy_enable;
+extern char *proxy_server;
+extern int auth_enable;
+extern int auth_enable_var;
+/*Basic Authentication Arguments*/
+char *basic_auth = NULL;
+
 void parse_options(int argc, char *argv[], playlist *pl)
 {
     struct option long_options[] =
@@ -63,7 +74,6 @@ void parse_options(int argc, char *argv[], playlist *pl)
         { "mono", 0, 0, 'm' },
         { "mix", 0, 0, 'm' },
         { "control", 0, 0, 'C' },
-        { "auth", 0, 0, 'u' },
     
         /* these take a parameter and have short equiv */
         { "doublespeed", 1, 0, 'd' },
@@ -71,6 +81,8 @@ void parse_options(int argc, char *argv[], playlist *pl)
         { "scale", 1, 0, 'f' },
         { "buffer", 1, 0, 'b' },
         { "proxy", 1, 0, 'p' },
+        { "auth", 1, 0, 'u' },
+        { "authvar", 1, 0, 'U' },
         { "rate", 1, 0, 'r' },
             
         /* The following are all implemented. */
@@ -102,6 +114,7 @@ void parse_options(int argc, char *argv[], playlist *pl)
         { "wav", 1, 0, 'w' },
         { "audiodevice", 1, 0, 'a' },
         { "gain", 1, 0, 'g' },
+        { "loop", 1, 0, 'l' },
         { 0, 0, 0, 0 }
     };
     int option_index = 0, c;
@@ -109,17 +122,29 @@ void parse_options(int argc, char *argv[], playlist *pl)
     options.maxframes=-1;
 
     while ((c = getopt_long(argc, argv, 
-                                "OPLTNEI824cy01mCu:d:h:f:b:p:r:G:" /* unimplemented */
-                                "A:D:vqtsVHzZRo:n:@:k:w:a:g:",     /* implemented */
+                                "OPLTNEI824cy01mCd:h:f:b:r:G:" /* unimplemented */
+                                "A:D:vqtsVHzZRo:n:@:k:w:a:g:l:p:u:U:",     /* implemented */
                         long_options, &option_index)) != -1)
     {            
         switch(c)
         {
             case 'O': case 'P': case 'L': case 'N': case 'E': case '8':
             case '2': case '4': case 'c': case 'y': case '0': case '1': case 'm': case 'C':
-            case 'u':
-            case 'U': case 'd': case 'h': case 'f': case 'b': case 'p':
+            case 'd': case 'h': case 'f': case 'b': 
                 break;
+	    case 'p':
+		proxy_server = strdup(optarg);
+		proxy_enable = 1;
+		break;
+	    case 'u':
+		basic_auth = strdup(optarg);
+		auth_enable = 1;
+		auth_enable_var = 0;
+		break;
+	    case 'U':
+		basic_auth = strdup(optarg);
+		auth_enable_var = 1;
+		auth_enable = 0;
             case 'n': 
                 options.maxframes = atol(optarg);
                 break;
@@ -238,9 +263,14 @@ void parse_options(int argc, char *argv[], playlist *pl)
                 break;    
             
             case 'g':
-                options.volume = atoi(optarg);
-                options.volume = (options.volume/100.0) * MAD_F_ONE;
-                break;
+                options.volume = mad_f_tofixed(atoi(optarg)/100.0);
+		break;
+
+            case 'l':
+             loop_remaining = atoi(optarg);
+              if (!loop_remaining)
+                 loop_remaining = -1;
+              break;
 
             case 'I':
                 /* We only try to get high priority. If it fails (just as
@@ -257,7 +287,8 @@ void parse_options(int argc, char *argv[], playlist *pl)
                 break;
 
             case 'V':
-                printf("mpg321 version " VERSION ". Copyright (C) 2001, 2002 Joe Drew.\n\n"
+                printf("mpg321 version " VERSION ". Copyright (C) 2001, 2002 Joe Drew,\n"
+		       "now maintained by Nanakos Chrysostomos and others.\n\n"
                        "This program is free software; you can redistribute it and/or modify\n"
                        "it under the terms of the GNU General Public License as published by\n"
                        "the Free Software Foundation; either version 2 of the License, or\n"
