@@ -27,9 +27,9 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
-#define _LARGEFILE_SOURCE 1
-
-#include "mpg321.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdlib.h>
 #include <string.h>
@@ -41,11 +41,13 @@
 #include <errno.h>
 #include <libgen.h>
 
+#include "mpg321.h"
+
 playlist * new_playlist()
 {
     playlist *pl = (playlist *) malloc(sizeof(playlist));
 
-    srandom(time(NULL));
+    srand(time(NULL));
         
     pl->files = (char **) malloc(DEFAULT_PLAYLIST_SIZE * sizeof(char *));
     pl->files_size = DEFAULT_PLAYLIST_SIZE;
@@ -87,25 +89,18 @@ void set_random_play(playlist *pl)
 
 void shuffle_files (playlist *pl)
 {
-    int i;
-    int a, b;
+    int i, r;
     char *swap;
 
-    for (i = 0; i < 100 * pl->numfiles; i++)
+    for (i = 0; i < pl->numfiles; i++)
     {
-        a = pl->numfiles * ((double)random()/RAND_MAX);
-        b = pl->numfiles * ((double)random()/RAND_MAX);
+        /* find a random integer r with i<=r<numfile */
+        r = i + (pl->numfiles-i-1) * ((double)rand()/RAND_MAX);
 
-        /* because if random()/RAND_MAX == 1, we will be 
-           referencing an index outside the array */
-        if (a == pl->numfiles)
-            a--;
-        if (b == pl->numfiles)
-            b--;
-
-        swap = pl->files[a];
-        pl->files[a] = pl->files[b];
-        pl->files[b] = swap;
+        /* now swap the current element with the random element */
+        swap = pl->files[i];
+        pl->files[i] = pl->files[r];
+        pl->files[r] = swap;
     }
 }
 
@@ -138,6 +133,12 @@ char * get_next_file(playlist *pl, buffer *buf)
     if (!pl->random_play)
     {
         if (i == pl->numfiles)
+        {
+            i = 0;
+            if (loop_remaining != -1)
+                loop_remaining--;
+        }
+        if (!loop_remaining)
             return NULL;
         
         return pl->files[i++];
@@ -148,10 +149,8 @@ char * get_next_file(playlist *pl, buffer *buf)
         if (!pl->numfiles)
             return NULL;
 
-        i = pl->numfiles * ((double)random()/RAND_MAX);
-        
-        if (i == pl->numfiles) i--;
-        
+        i = (pl->numfiles-1) * ((double)rand()/RAND_MAX);
+
         return pl->files[i];
     }
 }
@@ -215,7 +214,7 @@ void load_playlist(playlist *pl, char *filename)
     {
         trim_whitespace(playlist_buf);
 
-        if (strlen(playlist_buf) == 0)
+        if (strlen(playlist_buf) == 0 || playlist_buf[0] == '#')
             continue;                
 
         if (pl->numfiles == pl->files_size)
